/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include "buffer_array.h"
#include "jpg_obj.h"
#include "gif_obj.h"
#include "png_obj.h"
#include "url_list.h"
#include "file_obj.h"
#include "mime_types.h"
#include "parse.h"

buffer_t *buffer_array = NULL;

void reset_buffer_array(void)
{
	buffer_array = calloc(option_values.max_sim_conns, sizeof(buffer_t));
}

int get_buffer(CURL *id)
{
	int i;

	for (i = 0; buffer_array[i].id != id && i < option_values.max_sim_conns; i++);

	if (i >= option_values.max_sim_conns)
		return -1;

	return i;
}

static int get_buffer_by_url(url_list_t *url, int id)
{
	int i;

	if (!url)
		return -1;

	for (i = 0; i < option_values.max_sim_conns; i++)
		if (i != id && buffer_array[i].url && buffer_array[i].url->id == url->id)
			break;

	if (i >= option_values.max_sim_conns)
		return -1;

	return i;
}

int count_number_of_hostname(UriUriA *uri)
{
	int i, count = 0;

	if (!uri)
		return -1;

	for (i = 0; i < option_values.max_sim_conns; i++)
		if (buffer_array[i].uri) {
			if (host_is_equal(buffer_array[i].uri, uri))
				count++;
		}

	return count;
}

int is_uri_compatible(UriUriA *uri, int maxconns)
{
	int count = count_number_of_hostname(uri);

	return count >= 0 && count < (maxconns > 0 ? maxconns : option_values.max_sim_conns_per_host);
}

#ifdef ENABLE_METALINK
int open_buffer(CURL *id, url_list_t *url, UriUriA *uri, chunk_t *chunk, metalink_resource_list_t *resource,
	int header)
#else
int open_buffer(CURL *id, url_list_t *url, UriUriA *uri)
#endif
{
	int i;
	char *out_dir;

	if ((i = get_buffer(NULL)) < 0)
		return -1;

	buffer_array[i].id = id;
	buffer_array[i].url = url;
	buffer_array[i].uri = uri;
#ifdef ENABLE_METALINK
	buffer_array[i].chunk = chunk;
	buffer_array[i].used_res = resource;
#endif
	buffer_array[i].filename[MAX_FILENAME_LENGTH] = 0;

	out_dir = option_values.output_directory ? option_values.output_directory : DEFAULT_OUTPUT_DIRECTORY;
#ifdef ENABLE_METALINK
	if (chunk || header) {
		int id_file;

		snprintf(buffer_array[i].filename, MAX_FILENAME_LENGTH, "%s%smetalink-mulktmp%05d",
			out_dir, out_dir[strlen(out_dir)-1] == '/' ? "" : "/" , url->id);

		if (header || !is_file_exist(buffer_array[i].filename) || !url->tmp_file_created) {
			if (make_dir_pathname(buffer_array[i].filename, DEFAULT_MASK)) {
				remove(buffer_array[i].filename);
				buffer_array[i].file_pt = fopen(buffer_array[i].filename, "wb");
				if (!header && url->metalink_uri->size > 0)
					create_empty_file(&buffer_array[i], url->metalink_uri->size);
			}
		}
		else if ((id_file = get_buffer_by_url(url, i)) < 0) {
			if (make_dir_pathname(buffer_array[i].filename, DEFAULT_MASK))
				buffer_array[i].file_pt = fopen(buffer_array[i].filename, "rb+");
		}
		else {
			buffer_array[i].file_pt = buffer_array[id_file].file_pt;
		}
	}
	else
#endif /* ENABLE_METALINK */
	{
		snprintf(buffer_array[i].filename, MAX_FILENAME_LENGTH, "%s%smulktmp%05d",
			out_dir, out_dir[strlen(out_dir)-1] == '/' ? "" : "/" , i);

		if (make_dir_pathname(buffer_array[i].filename, DEFAULT_MASK))
			buffer_array[i].file_pt = fopen(buffer_array[i].filename, "wb");
	}

	if (is_printf(MINFO)) {
		char *uri_str = uri2string(uri);
		printf("Open link #%d, url: %s, tmpfile: %s\n", i, uri_str ? uri_str : "", buffer_array[i].filename);
		if (uri_str)
			free(uri_str);
	}

	return i;
}

static void filter_buffer(int i, char *base_url)
{
	char *out_dir;
	char newfilename[MAX_FILENAME_LENGTH+1];
	char newimagename[MAX_FILENAME_LENGTH+1];
	int len;

#ifdef ENABLE_METALINK
	if (buffer_array[i].url->metalink_uri 
		&& !is_file_downloaded(buffer_array[i].url->metalink_uri))
		return;
#endif

	if (!base_url) {
		remove(buffer_array[i].filename);
		return;
	}

	newfilename[0] = 0;
	newimagename[0] = 0;
	newfilename[MAX_FILENAME_LENGTH] = 0;
	newimagename[MAX_FILENAME_LENGTH] = 0;

	if (is_html_file(buffer_array[i].url->mimetype)) 
		parse_urls(buffer_array[i].filename, base_url, buffer_array[i].url->level);

#ifdef ENABLE_METALINK
	if (buffer_array[i].url->metalink_uri) {
		strncpy(newfilename, buffer_array[i].url->metalink_uri->file->name, MAX_FILENAME_LENGTH);
	}
	else 
#endif /* ENABLE_METALINK */
	if (!option_values.disable_save_tree) {
		UriUriA *uri = create_absolute_uri(NULL, base_url);
		char *furi_str = uri2filename(uri);

		if (furi_str) {
			strncpy(newfilename, furi_str, MAX_FILENAME_LENGTH);

			len = strlen(newfilename);
			if (newfilename[len-1] == '/' && is_html_file(buffer_array[i].url->mimetype)) 
				strncat(newfilename, "index.html", MAX_FILENAME_LENGTH - len);
		}

		if (furi_str)
			free(furi_str);
		uri_free(uri);
	}

	if (is_gif_image(buffer_array[i].url->mimetype) && is_valid_gif_image(buffer_array[i].filename)) {
		out_dir = option_values.output_directory ? option_values.output_directory :
			DEFAULT_OUTPUT_DIRECTORY;
		snprintf(newimagename, MAX_FILENAME_LENGTH, "%s%sgif-image%05d.gif",
			out_dir, out_dir[strlen(out_dir)-1] == '/' ? "" : "/" , buffer_array[i].url->id);
	}
	if (is_png_image(buffer_array[i].url->mimetype) && is_valid_png_image(buffer_array[i].filename)) {
		out_dir = option_values.output_directory ? option_values.output_directory :
			DEFAULT_OUTPUT_DIRECTORY;
		snprintf(newimagename, MAX_FILENAME_LENGTH, "%s%spng-image%05d.png",
			out_dir, out_dir[strlen(out_dir)-1] == '/' ? "" : "/" , buffer_array[i].url->id);
	}
	if (is_jpeg_image(buffer_array[i].url->mimetype) && is_valid_jpeg_image(buffer_array[i].filename)) {
		out_dir = option_values.output_directory ? option_values.output_directory :
			DEFAULT_OUTPUT_DIRECTORY;
		snprintf(newimagename, MAX_FILENAME_LENGTH, "%s%sjpg-image%05d.jpeg",
			out_dir, out_dir[strlen(out_dir)-1] == '/' ? "" : "/" , buffer_array[i].url->id);
	}

	if (*newfilename && *newimagename) {
		if (is_printf(MINFO)) {
			printf("Saving file: %s\n", newfilename);
			printf("Saving file: %s\n", newimagename);
		}

		if (make_dir_pathname(newfilename, DEFAULT_MASK))
			copy(buffer_array[i].filename, newfilename);
		if (make_dir_pathname(newimagename, DEFAULT_MASK))
			rename(buffer_array[i].filename, newimagename);
		else
			remove(buffer_array[i].filename);
	}
	else if (*newfilename) {
		if (is_printf(MINFO))
			printf("Saving file: %s\n", newfilename);

		if (make_dir_pathname(newfilename, DEFAULT_MASK))
			rename(buffer_array[i].filename, newfilename);
		else
			remove(buffer_array[i].filename);
	}
	else if (*newimagename) {
		if (is_printf(MINFO))
			printf("Saving file: %s\n", newimagename);

		if (make_dir_pathname(newimagename, DEFAULT_MASK))
			rename(buffer_array[i].filename, newimagename);
		else
			remove(buffer_array[i].filename);
	}
	else
		remove(buffer_array[i].filename);
}

void set_buffer_mime_type(CURL *id, char *mimetype)
{
	int i;

	if ((i = get_buffer(id)) < 0)
		return;

	buffer_array[i].url->mimetype[MAX_MIMETYPE_LENGTH] = 0;
	
	if (mimetype)
		strncpy(buffer_array[i].url->mimetype, mimetype, MAX_MIMETYPE_LENGTH);
}

int create_empty_file(buffer_t *buffer, size_t size)
{
	int ret;

	if (buffer->file_pt)
		fclose(buffer->file_pt);
	else
		return -1;

	buffer->file_pt = NULL;

	if ((ret = truncate(buffer->filename, size)))
		return ret;

	buffer->file_pt = fopen(buffer->filename, "rb+");

	buffer->url->tmp_file_created = 1;
	return 0;
}

#ifdef ENABLE_METALINK
static int is_valid_response(CURLcode err_code, long resp_code, chunk_t *chunk)
{
	return !err_code && ((resp_code == 200 && chunk == NULL) || (resp_code == 206 && chunk && chunk->pos == chunk->length));
}
#else /* not ENABLE_METALINK */
static int is_valid_response(CURLcode err_code, long resp_code)
{
	return !err_code && resp_code == 200;
}
#endif /* not ENABLE_METALINK */

void close_buffer(CURL *id, char *base_url, CURLcode err_code, long resp_code)
{
	int i, id_file;
#ifdef ENABLE_METALINK
	double length_double;
	long length;
	long byte_downloaded = 0, byte_total = 0;
	int chunk_completed = 0, chunk_total = 0;
#endif

	if ((i = get_buffer(id)) < 0)
		return;

#ifdef ENABLE_METALINK
	if (buffer_array[i].url->metalink_uri) {
		if (buffer_array[i].url->metalink_uri->size < 0 && is_valid_response(err_code, resp_code, NULL)) {
			curl_easy_getinfo(id, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &length_double);
			length = (long) length_double;
			buffer_array[i].url->metalink_uri->size = length;

			if (length) {
				create_empty_file(&buffer_array[i], length);
				create_chunks(buffer_array[i].url->metalink_uri);
			}
		}

		buffer_array[i].url->metalink_uri->header = 0;
	}
#endif /* ENABLE_METALINK */

	if ((id_file = get_buffer_by_url(buffer_array[i].url, i)) < 0) {
		if (buffer_array[i].file_pt)
			fclose(buffer_array[i].file_pt);

		filter_buffer(i,
#ifdef ENABLE_METALINK
			!is_valid_response(err_code, resp_code, buffer_array[i].chunk)
#else
			!is_valid_response(err_code, resp_code)
#endif
			? NULL : base_url);
	}

	if (is_printf(MINFO))
		printf("Close link #%d\n", i);
	
#ifdef ENABLE_METALINK
	if (buffer_array[i].chunk) {
		if (buffer_array[i].chunk->used_res) {
			buffer_array[i].chunk->used_res->assigned--;
			if (buffer_array[i].chunk->used_res->assigned < 0)
				buffer_array[i].chunk->used_res->assigned = 0;
		}
		buffer_array[i].chunk->used_res = NULL;
		if (!is_valid_response(err_code, resp_code, buffer_array[i].chunk))
			buffer_array[i].chunk->pos = 0;

		if (file_statistics(buffer_array[i].url->metalink_uri, &chunk_completed, &chunk_total,
			&byte_downloaded, &byte_total)) {
			if (is_printf(MINFO))
				printf("Downloaded %d/%d chunks, %ld/%ld bytes\n", chunk_completed, chunk_total,
					byte_downloaded, byte_total);
		}
	}

	if (!is_valid_response(err_code, resp_code, buffer_array[i].chunk)
		&& buffer_array[i].url->metalink_uri && buffer_array[i].used_res)
		remove_metalink_resource(buffer_array[i].url->metalink_uri, buffer_array[i].used_res);

	if (buffer_array[i].chunk && chunk_completed == chunk_total && chunk_total) {
		if (is_printf(MNOTE))
			printf("RESULT: Metalink downloaded successfully, Filename:\"%s\" Size:%ld\n",
				buffer_array[i].url->metalink_uri->file->name, byte_downloaded);
	}
	else if (buffer_array[i].url->metalink_uri && !is_resource_available(buffer_array[i].url->metalink_uri)) {
		if (is_printf(MNOTE))
			printf("RESULT: Metalink error, no more useful resources to download, Filename:\"%s\"\n",
				buffer_array[i].url->metalink_uri->file->name);
	}
#endif /* ENABLE_METALINK */

	buffer_array[i].file_pt = NULL;
	buffer_array[i].id = NULL;
	buffer_array[i].url = NULL;
	buffer_array[i].uri = NULL;
#ifdef ENABLE_METALINK
	buffer_array[i].chunk = NULL;
	buffer_array[i].used_res = NULL;
#endif
}

void print_buffers(void)
{
	int i;

	if (is_printf(MINFO)) {
		for (i = 0; i < option_values.max_sim_conns; i++)
		{
#ifdef ENABLE_METALINK
			char *uri_str = uri2string(buffer_array[i].used_res->uri);
			printf("%d: %s, %s, %s", i, buffer_array[i].id ? "PRESENT" : "EMPTY", 
				buffer_array[i].used_res ? uri_str : "NULL",
				buffer_array[i].chunk ? "CHUNK" : "NO CHUNK");
			if (uri_str)
				free(uri_str);
#else /* not ENABLE_METALINK */
			printf("%d: %s", i, buffer_array[i].id ? "PRESENT" : "EMPTY");
#endif /* not ENABLE_METALINK */
			printf("\n");
		} 
	}
}

void free_buffer_array(void)
{
	free(buffer_array);
}

