/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#ifndef _BUFFER_ARRAY_H_
#define _BUFFER_ARRAY_H_

#include "defines.h"
#ifdef ENABLE_METALINK
#include "metalink_list.h"
#endif
#include "url_list.h"

#define MAX_FILENAME_LENGTH 1000
#define DEFAULT_OUTPUT_DIRECTORY "data"

/* file struct */
typedef struct buffer_t {
	CURL *id;
	FILE *file_pt;
	url_list_t *url;
	UriUriA *uri;
	char filename[MAX_FILENAME_LENGTH+1];

#ifdef ENABLE_METALINK
	metalink_resource_list_t* used_res;
	chunk_t *chunk;
#endif
} buffer_t;

/* url struct */
extern buffer_t *buffer_array;

void reset_buffer_array(void);

#ifdef ENABLE_METALINK
int open_buffer(CURL *id, url_list_t *url, UriUriA *uri, chunk_t *chunk, metalink_resource_list_t *resource,
	int header);
#else
int open_buffer(CURL *id, url_list_t *url, UriUriA *uri);
#endif

int get_buffer(CURL *id);

int count_number_of_hostname(UriUriA *uri);

int is_uri_compatible(UriUriA *uri, int maxconns);

int create_empty_file(buffer_t *buffer, size_t size);

void close_buffer(CURL *id, char *base_url, CURLcode err_code, long resp_code);

void print_buffers(void);

void free_buffer_array(void);

void set_buffer_mime_type(CURL *id, char *mimetype);

#endif /* not _BUFFER_ARRAY_H_ */
