/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#include "file_obj.h"
#include "url_list.h"

#define MAX_PATH_LEN 2000
#define BUFFER_SIZE 1024

int make_dir_pathname(char *pathname, int arg_mode)
{
	struct stat stat_buf;
	char *slash_ptr;
	char path[MAX_PATH_LEN+1];

	path[MAX_PATH_LEN] = 0;
	strncpy(path, pathname, MAX_PATH_LEN+1);
	if (path[MAX_PATH_LEN] != 0)
		return 0;

	arg_mode |= (S_IWUSR | S_IXUSR);

	slash_ptr = strchr(path, '/');

	while (slash_ptr) {
		*slash_ptr = 0;

		if (stat(path, &stat_buf) < 0) {
			if (mkdir(path, arg_mode) < 0)
				return 0;
		}
		else if (!S_ISDIR(stat_buf.st_mode))
			return 0;

		*slash_ptr = '/';
		slash_ptr = strchr(slash_ptr + 1, '/');
	}

	return 1;
}

int copy(char *from_filename, char *to_filename)
{
	char buffer[BUFFER_SIZE];
	FILE *fp_to, *fp_from;
	int ret = -1, count;

	if (!(fp_from = fopen(from_filename, "rb")))
		return -1;

	if (!(fp_to = fopen(to_filename, "wb")))
		goto Exit1;

	while ((count = fread(buffer, 1, BUFFER_SIZE, fp_from)))
		if (fwrite(buffer, 1, count, fp_to) < count)
			goto Exit;

	ret = (ferror(fp_to) || ferror(fp_from)) ? -1 : 0;

Exit:
	fclose(fp_to);
Exit1:
	fclose(fp_from);

	return ret;
}

size_t fwrite_offset(char *buf, size_t size, size_t nmemb, size_t offset, FILE *file)
{
	fseek(file, offset, SEEK_SET); 

	return fwrite(buf, size, nmemb, file);
}

void trim(char *buf)
{
	char *from, *to;

	for (from = buf, to = buf; *from; from++) 
		if (*from != ' ' && *from != '\r' && *from != '\n' && *from != '\t')  
			*(to++) = *from; 

	*to = 0;
}

void strmove(char *to, char *from)
{
	if (!to || !from)
		return;

	while (*from) 
		*(to++) = *(from++); 

	*to = 0;
}

void replacestr_with_char(char *buf, const char *str, const char ch)
{
	char *ptr;

	if (!buf || !str)
		return;

	while ((ptr = strstr(buf, str))) {
		*ptr = ch;
		
		strmove(ptr + 1, ptr + strlen(str));
	}
}

void read_uri_from_text_file(char *filename)
{
	FILE *file;
	char buf[MAX_LINE_LENGTH];

	if (!(file = fopen(filename, "r")))
		return;

	while (1) {
		if (!fgets(buf, MAX_LINE_LENGTH, file))
			break;
		if (feof(file))
			break;

		trim(buf);

		/* skip empty lines and comments */
		if (!strlen(buf) || *buf == '#')
			continue;
 
		if (is_printf(MNOTE))
			printf("Add url to download from text file: %s\n", buf);
		push_unique_url(NULL, buf, 1);
	}

	fclose(file);
}

/* read only long option */
int read_option_from_text_file(char *filename)
{
	FILE *file;
	char buf[MAX_LINE_LENGTH], *option, *value;
	int value_used, ret = 0;

	if (!(file = fopen(filename, "r")))
		return 1;

	while (1) {
		if (!fgets(buf, MAX_LINE_LENGTH, file))
			break;
		if (feof(file))
			break;

		trim(buf);

		/* skip empty lines and comments */
		if (!strlen(buf) || *buf == '#')
			continue;

		option = buf;
		if ((value = strchr(buf, '='))) {
			*value = 0;
			value++;
			if (!*value)
				value = NULL;
		}
		else
			value = NULL;

		if (is_printf(MNOTE))
			printf("Add option from text file: %s%s%s\n", option,
				value ? "=" : "", value ? value : "");
	
		if ((ret = set_option(option, value, 1, &value_used)))
			break;
	}

	fclose(file);

	return ret;
}

int is_file_exist(char *filename)
{
	struct stat sts;

	if (stat(filename, &sts) == -1 && errno == ENOENT)
		return 0;
	return 1;
}
