/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "gif_obj.h"

#define GIF_HEADER_SIZE 6
#define GIF_DIM_SIZE 4

#define GIF_HEADER_VAL1 "GIF87a"
#define GIF_HEADER_VAL2 "GIF89a"

#define MAKE_WORD(h, l) (((h) << 8) | (l))

static int read_gif_dim(char *filename, int *width, int *height)
{
	unsigned char buffer[GIF_HEADER_SIZE + GIF_DIM_SIZE], *dim;
	int ret = 0, bytestoread;
	FILE *file = NULL;

	if (!(file = fopen(filename, "rb")))
		return 0;

	bytestoread = GIF_HEADER_SIZE + GIF_DIM_SIZE;
	if (fread(buffer, 1, bytestoread, file) != bytestoread)
		goto Exit;

	if (memcmp(buffer, GIF_HEADER_VAL1, GIF_HEADER_SIZE)
		&& memcmp(buffer, GIF_HEADER_VAL2, GIF_HEADER_SIZE))
		goto Exit;

	dim = buffer + GIF_HEADER_SIZE;
	*width = MAKE_WORD(dim[1], dim[0]);
	*height = MAKE_WORD(dim[3], dim[2]);

	ret = 1;

Exit:
	fclose(file);
	return ret;
}

int is_valid_gif_image(char *filename)
{
	int width = 0, height = 0, valid;

	if (!option_values.save_gif_image)
		return 0;

	valid = read_gif_dim(filename, &width, &height);

	valid = valid && (width >= option_values.min_image_width) 
		&& (!option_values.max_image_width || width <= option_values.max_image_width) 
		&& (height >= option_values.min_image_height) 
		&& (!option_values.max_image_height || height <= option_values.max_image_height);

	if (is_printf(MINFO)) 
		printf("Image GIF %s: %s, width = %d, height = %d\n", valid ? "OK" : "FILTERED",
			filename, width, height);

	return valid;
}
