/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#ifndef _METALINK_LIST_H_
#define _METALINK_LIST_H_

#include <metalink/metalink_parser.h>
#include "defines.h"

#define USED_TYPE "http"

#define CHUNK_NUMBER 20L
#define MIN_CHUNK_SIZE 30000L
#define MAX_CHUNK_SIZE 3000000L

typedef struct metalink_resource_list_t {
	struct metalink_resource_list_t *prev, *next;
	metalink_resource_t* resource;
	UriUriA *uri;
	int assigned;
	int error;
} metalink_resource_list_t;

struct metalink_file_list_t;

typedef struct chunk_t {
	struct chunk_t *next;
	struct metalink_file_list_t *file;
	metalink_resource_list_t* used_res;
	long start;
	long length;
	long pos;
} chunk_t;

typedef struct metalink_file_list_t {
	metalink_file_t *file;
	long size;
	int header;
	metalink_resource_list_t *usable_res_top;
	metalink_resource_list_t *usable_res_bottom;
	int chunk_number;
	chunk_t *chunk;
} metalink_file_list_t;

typedef struct metalink_list_t {
	struct metalink_list_t *next;
	metalink_t* metalink;
} metalink_list_t;


void push_chunk(metalink_file_list_t *file, long start, long length);

void print_chunks(metalink_file_list_t *file);

chunk_t *find_free_chunk(metalink_file_list_t *file);

int is_file_downloaded(metalink_file_list_t *file);

int file_statistics(metalink_file_list_t *file, int *chunk_completed, int *chunk_total,
	long *byte_downloaded, long *byte_total);

void create_chunks(metalink_file_list_t *file);

void remove_metalink_resource(metalink_file_list_t *file, metalink_resource_list_t *resource);

int is_resource_available(metalink_file_list_t *file);

void free_resources(metalink_file_list_t *file);

metalink_file_list_t *create_metalink_file(metalink_file_t *file);

UriUriA *find_next_url(metalink_file_list_t *file, chunk_t **chunk, metalink_resource_list_t **resource, 
	int *header);

void set_metalink_file_length(metalink_file_list_t *file, long size);

void free_metalink_file(metalink_file_list_t *file);

void push_metalink(metalink_t *metalink);

void free_metalinks(void);

void add_new_metalink(char *filename, int level);

int save_locations(char *loc_option);

#endif /* not _METALINK_LIST_H_ */
