/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#include <ctype.h>
#include "mime_types.h"

int strcmplower(char *str1, char *str2)
{
	for (; *str1 && *str2 && (tolower(*str1) == tolower(*str2)); str1++, str2++);

	return (!*str1 && !*str2) ? 0 : ((tolower(*str1) < tolower(*str2)) ? -1 : 1);
}

int strncmplower(char *str1, char *str2, int len)
{
	int i;

	for (i = 0; i < len && *str1 && *str2 && (tolower(*str1) == tolower(*str2)); str1++, str2++, i++);

	return (i >= len) ? 0 : ((tolower(*str1) < tolower(*str2)) ? -1 : 1);
}

char *strstrlower(char *str1, char *str2)
{
	int i, len1 = strlen(str1), len2 = strlen(str2);

	for (i = 0; i <= len1 - len2; i++)
		if (!strncmplower(str1 + i, str2, len2))
			return str1 + i;

	return NULL;
}

int is_same_mimetype(char *type1, char *type2)
{
	return !strncmplower(type1, type2, strlen(type2));
}

int is_gif_image(char *type)
{
	return is_same_mimetype(type, TYPE_GIF);
}

int is_png_image(char *type)
{
	return is_same_mimetype(type, TYPE_PNG);
}

int is_jpeg_image(char *type)
{
	return is_same_mimetype(type, TYPE_JPEG) || is_same_mimetype(type, TYPE_PJPEG);
}

int is_html_file(char *type)
{
	return is_same_mimetype(type, TYPE_HTML);
}
