/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#include "defines.h"
#include "file_obj.h"


char *uri2string(UriUriA *uri)
{
	char *uri_str;
	int length;

	if (!uri)
		return NULL;

	if (uriToStringCharsRequiredA(uri, &length) != URI_SUCCESS)
		return NULL;

	length++;
	if (!(uri_str = calloc(1, length * sizeof(char))))
		return NULL;

	if (uriToStringA(uri_str, uri, length, NULL) != URI_SUCCESS) {
		free(uri_str);
		return NULL;
	}

	return uri_str;
}

char *uri2filename(UriUriA *uri)
{
	char *uri_str = uri2string(uri);

	if (!uri_str)
		return NULL;

	replacestr_with_char(uri_str, "//", '/');

	return uri_str;
}

static UriUriA *uri_alloc_1(void)
{
	return calloc(1, sizeof(UriUriA));
}

void uri_free(UriUriA *uri)
{
	if (uri) {
		uriFreeUriMembersA(uri);
		free(uri); 
	}
}

UriUriA *create_absolute_uri(char *base_url, char *url)
{
	UriParserStateA state;
	UriUriA *abs_dest, abs_base, rel_source;

	if (!url || !*url)
		return NULL;

	if (!(abs_dest = uri_alloc_1()))
		return NULL;
	if (base_url) {
		state.uri = &rel_source;
		if (uriParseUriA(&state, url) != URI_SUCCESS) {
			free(abs_dest); 
			uriFreeUriMembersA(&rel_source);
			return NULL;
		}

		state.uri = &abs_base;
		if (uriParseUriA(&state, base_url) != URI_SUCCESS) {
			free(abs_dest); 
			uriFreeUriMembersA(&abs_base);
			uriFreeUriMembersA(&rel_source);
			return NULL;
		}

		if (uriAddBaseUriA(abs_dest, &rel_source, &abs_base) != URI_SUCCESS) {
			uri_free(abs_dest);
			uriFreeUriMembersA(&abs_base);
			uriFreeUriMembersA(&rel_source);
			return NULL;
		}

		uriFreeUriMembersA(&abs_base);
		uriFreeUriMembersA(&rel_source);
	}
	else {
		state.uri = abs_dest;
		if (uriParseUriA(&state, url) != URI_SUCCESS) {
			uri_free(abs_dest); 
			return NULL;
		}
	}

	if (uriNormalizeSyntaxA(abs_dest) != URI_SUCCESS) {
		uri_free(abs_dest);
		return NULL;
	}

	return abs_dest;
}

int host_is_equal(UriUriA *uri1, UriUriA *uri2)
{
	return (uri1->hostText.first && uri2->hostText.first 
		&& (uri1->hostText.afterLast - uri1->hostText.first) == (uri2->hostText.afterLast - uri2->hostText.first) 
		&& !strncmp(uri1->hostText.first, uri2->hostText.first, uri1->hostText.afterLast - uri1->hostText.first));
}
