/*---------------------------------------------------------------------------
 * Copyright (C) 2008-2009 - Emanuele Bovisio
 *
 * This file is part of Mulk.
 *
 * Mulk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Mulk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mulk.  If not, see <http://www.gnu.org/licenses/>.
 *---------------------------------------------------------------------------*/

#ifndef _URL_LIST_H_
#define _URL_LIST_H_

#include "defines.h"
#ifdef ENABLE_METALINK
#include "metalink_list.h"
#endif

#define MAX_MIMETYPE_LENGTH 200

typedef struct url_list_t {
	int id;
	int level;
	int assigned;
	int tmp_file_created;
	UriUriA *uri;
	char mimetype[MAX_MIMETYPE_LENGTH+1];
	struct url_list_t *next;
	
#ifdef ENABLE_METALINK
	/* metalink */
	metalink_file_list_t *metalink_uri;
#endif
} url_list_t;

extern url_list_t *top;
extern url_list_t *bottom;
extern url_list_t *pointer;

UriUriA *create_absolute_url(char *base_url, char *url);

void push_uri(UriUriA *uri, int level);
void push_url(char *base_url, char *url, int level);
void push_unique_url(char *base_url, char *url, int level);

url_list_t *get_next(url_list_t *next);
#ifdef ENABLE_METALINK
url_list_t *search_next_url(UriUriA **uri, chunk_t **chunk, metalink_resource_list_t **resource,
	int *header);

void push_metalink_uri(metalink_file_list_t *metalink_uri, int level);

void set_url_file_length(url_list_t *url, long size);
#else /* not ENABLE_METALINK */
url_list_t *search_next_url(UriUriA **uri);
#endif /* not ENABLE_METALINK */

/* remember to free the url */
UriUriA *pop_url(void);

void print_urls(char *filename);
void print_csv_urls(char *filename);

void free_urls(void);

#endif /* not _URL_LIST_H_ */
